#include "mm.h"

static sboolean _s_mm_initialized = FALSE;
static SMMType _s_mm_type = S_MM_TYPE_NONE;

void
s_mm_init (SMMType type) {
  if (!_s_mm_initialized) {
    _s_mm_initialized = TRUE;
    #if USE_GC
    _s_mm_type = type;
    #else
    _s_mm_type = S_MM_TYPE_LIBC;
    #endif
  }
}

schar *
s_mm_type_get_name (SMMType type) {
  return SMMTypeName[type];
}

void
s_mm_cleanup () {
  /* Nothing... yet. */
}

SMMType
s_mm_get_type () {
  return _s_mm_type;
}

spointer
s_malloc (size_t size) {
  #if USE_GC
  if (_s_mm_type == S_MM_TYPE_GC) {
    return GC_MALLOC (size);
  }
  #endif
  return malloc (size);
}

void
s_free (spointer pointer) {
  #if USE_GC
  if (_s_mm_type == S_MM_TYPE_GC) {
    /* Because libgc handles the freeing of these things, we just set it to
     * NULL and be done with it.
     *
     * This also means that we can used this as a normal free and not have to
     * deal with setting it to NULL.
     */
    pointer = NULL;
    return;
  }
  #endif
  free (pointer);
}

spointer
s_realloc (spointer pointer, size_t size) {
  #if USE_GC
  if (_s_mm_type == S_MM_TYPE_GC) {
    return GC_REALLOC (pointer, size);
  }
  #endif
  return realloc (pointer, size);
}

spointer
s_calloc (size_t num, size_t size) {
  #if USE_GC
  if (_s_mm_type == S_MM_TYPE_GC) {
    /* This should work. */
    spointer ret_val = GC_MALLOC (num * size);
    return ret_val;
  }
  #endif
  return calloc (num, size);
}
